
typedef union  {
	Node	*p;
	Cell	*cp;
	int	i;
	char	*s;
}	YYSTYPE;
extern	YYSTYPE	yylval;
#define	FIRSTTOKEN	57346
#define	PROGRAM	57347
#define	PASTAT	57348
#define	PASTAT2	57349
#define	XBEGIN	57350
#define	XEND	57351
#define	NL	57352
#define	ARRAY	57353
#define	MATCH	57354
#define	NOTMATCH	57355
#define	MATCHOP	57356
#define	FINAL	57357
#define	DOT	57358
#define	ALL	57359
#define	CCL	57360
#define	NCCL	57361
#define	CHAR	57362
#define	OR	57363
#define	STAR	57364
#define	QUEST	57365
#define	PLUS	57366
#define	EMPTYRE	57367
#define	AND	57368
#define	BOR	57369
#define	APPEND	57370
#define	EQ	57371
#define	GE	57372
#define	GT	57373
#define	LE	57374
#define	LT	57375
#define	NE	57376
#define	IN	57377
#define	ARG	57378
#define	BLTIN	57379
#define	BREAK	57380
#define	CLOSE	57381
#define	CONTINUE	57382
#define	DELETE	57383
#define	DO	57384
#define	EXIT	57385
#define	FOR	57386
#define	FUNC	57387
#define	SUB	57388
#define	GSUB	57389
#define	IF	57390
#define	INDEX	57391
#define	LSUBSTR	57392
#define	MATCHFCN	57393
#define	NEXT	57394
#define	NEXTFILE	57395
#define	ADD	57396
#define	MINUS	57397
#define	MULT	57398
#define	DIVIDE	57399
#define	MOD	57400
#define	ASSIGN	57401
#define	ASGNOP	57402
#define	ADDEQ	57403
#define	SUBEQ	57404
#define	MULTEQ	57405
#define	DIVEQ	57406
#define	MODEQ	57407
#define	POWEQ	57408
#define	PRINT	57409
#define	PRINTF	57410
#define	SPRINTF	57411
#define	ELSE	57412
#define	INTEST	57413
#define	CONDEXPR	57414
#define	POSTINCR	57415
#define	PREINCR	57416
#define	POSTDECR	57417
#define	PREDECR	57418
#define	VAR	57419
#define	IVAR	57420
#define	VARNF	57421
#define	CALL	57422
#define	NUMBER	57423
#define	STRING	57424
#define	REGEXPR	57425
#define	GETLINE	57426
#define	RETURN	57427
#define	SPLIT	57428
#define	SUBSTR	57429
#define	WHILE	57430
#define	CAT	57431
#define	NOT	57432
#define	UMINUS	57433
#define	POWER	57434
#define	DECR	57435
#define	INCR	57436
#define	INDIRECT	57437
#define	LASTTOKEN	57438
